#
# FOURJS_START_COPYRIGHT(U,2000)
# Property of Four Js*
# (c) Copyright Four Js 2000, 2012. All Rights Reserved.
# * Trademark of Four Js Development Tools Europe Ltd
#   in the United States and elsewhere
# 
# Four Js and its suppliers do not warrant or guarantee that these samples
# are accurate and suitable for your purposes. Their inclusion is purely for
# information purposes only.
# FOURJS_END_COPYRIGHT
#

IMPORT com
IMPORT xml

# 
# USER GLOBALS VARIABLES
#

GLOBALS

  DEFINE
    dom_in RECORD
        document xml.DomDocument ATTRIBUTE(XMLAny, XMLNamespace="##any") 
    END RECORD,
    dom_out RECORD
        document xml.DomDocument ATTRIBUTE(XMLAny, XMLNamespace="##any")
    END RECORD
    
END GLOBALS

DEFINE serverURL STRING
DEFINE screen    BOOLEAN          

&define display_status(status) \
  IF NOT screen THEN \
    DISPLAY status \
  ELSE \
    DISPLAY status TO MSG \
    MENU ON idle 1 EXIT MENU ON ACTION close EXIT PROGRAM END MENU \
  END IF

#
# MAIN
#
MAIN
  DEFINE ret INTEGER  
      
  DEFER INTERRUPT

  LET screen = false
  #
  # Check arguments
  #
  IF num_args() = 2 and arg_val(1) = "-W" THEN
      LET serverURL = arg_val(2)
      CALL CreateCalculatorService(true)
      EXIT PROGRAM
  ELSE 
    IF num_args() = 2 and arg_val(1) = "-S" THEN
      LET screen = true
      CALL fgl_setenv("FGLAPPSERVER",arg_val(2))
      CLOSE WINDOW SCREEN
      OPEN WINDOW w WITH FORM "server" ATTRIBUTES(TEXT = "Calculator service", STYLE="naked")
      display_status("Calculator service startup")
    ELSE
      IF num_args() <> 0 THEN
        CALL exitHelp()
        EXIT PROGRAM    
      END IF
    END IF
  END IF
  
  #
  # Create Calculator service
  #
  CALL CreateCalculatorService(false)
    
  #
  # Start the server
  #
  display_status("Starting server...")
  #
  # Starts the server on the port number specified by the FGLAPPSERVER environment variable
  #  (EX: FGLAPPSERVER=8090)
  # 
  CALL com.WebServiceEngine.Start()
  display_status("The server is listening.")

  IF screen THEN
    MENU
      ON idle 1
        LET ret = com.WebServiceEngine.ProcessServices(1)
        CASE ret
          WHEN 0
            DISPLAY "Request processed." TO msg
          WHEN -1
            DISPLAY "No request..." TO msg
          WHEN -2
            DISPLAY "Disconnected from application server." TO msg
            EXIT PROGRAM   # The Application server has closed the connection
          WHEN -3
            DISPLAY "Client Connection lost." TO msg
          WHEN -4
            DISPLAY "Server interrupted with Ctrl-C." TO msg
          WHEN -10
            DISPLAY "Internal server error." TO msg
        END CASE
      ON ACTION close
        EXIT program
    END MENU
  ELSE
    WHILE TRUE
      LET ret = com.WebServiceEngine.ProcessServices(-1)
      CASE ret
        WHEN 0
          DISPLAY "Request processed." 
        WHEN -1
          DISPLAY "Timeout reached."
        WHEN -2
          DISPLAY "Disconnected from application server."
          EXIT PROGRAM   # The Application server has closed the connection
        WHEN -3
          DISPLAY "Client Connection lost."
        WHEN -4
          DISPLAY "Server interrupted with Ctrl-C."
        WHEN -10
          DISPLAY "Internal server error."
      END CASE
      IF int_flag<>0 THEN
        LET int_flag=0
        EXIT WHILE
      END IF     
    END WHILE
    DISPLAY "Server stopped"
  END IF
END MAIN

#
# Create Calculator RPC/Literal service
#
FUNCTION CreateCalculatorService(generateWSDL)
  DEFINE serv         com.WebService       # WebService
  DEFINE op           com.WebOperation     # Operation of a WebService
  DEFINE serviceNS    STRING
  DEFINE generateWSDL SMALLINT
  DEFINE ret          INTEGER

  LET serviceNS       = "http://tempuri.org/"

  TRY
  
    #
    # Create Calculator Web Service
    #
    LET serv = com.WebService.CreateWebService("Calculator",serviceNS)
    CALL serv.setFeature("Soap1.1",TRUE)
    CALL serv.setFeature("Soap1.2",TRUE)
 
    # DOM
    --NOTE: DocStyle not RPC
    LET op = com.WebOperation.CreateDocStyle("dom","Dom",dom_in,dom_out)
    CALL serv.publishOperation(op,NULL)
  
    IF generateWSDL THEN
      #
      # Generate WSDL
      #
      LET ret = serv.saveWSDL(serverURL)
      IF ret=0 THEN
        DISPLAY "WSDL saved"      
      ELSE
        DISPLAY "ERROR: Unable to save WSDL"
      END IF
    ELSE
      #
      # Register service  
      #
      CALL com.WebServiceEngine.RegisterService(serv)  
      display_status("Calculator Service registered")
    END IF
    
  CATCH
    display_status("Unable to create 'Calculator' Web Service :"||STATUS)
    EXIT PROGRAM
  END TRY
    
END FUNCTION



FUNCTION exitHelp()
  DISPLAY "Usage: "
  DISPLAY "  ", arg_val(0)
  DISPLAY "    Start the server on port defined by FGLAPPSERVER"
  DISPLAY "  ", arg_val(0), " -W serverurl"
  DISPLAY "    Generate the WSDL file for the given url"
  DISPLAY "  ", arg_val(0), " -S port"
  DISPLAY "    Start service in graphical mode and on given port"
  EXIT PROGRAM
END FUNCTION


#
# USER PUBLIC FUNCTIONS
#



FUNCTION dom()
DEFINE root_in, root_out xml.DomNode
DEFINE a,b,r,m INTEGER
DEFINE op STRING

    -- Extract from XML Document
    LET root_in = dom_in.document.getDocumentElement()
    LET a = root_in.getAttribute("a")
    LET b = root_in.getAttribute("b")
    LET op = root_in.getAttribute("op")

    -- Do the maths and prepare the Document to Return
    LET dom_out.document = dom_in.document
    LET root_out = dom_out.document.getDocumentElement()

    CALL root_out.setAttribute("error","Ok")
    CASE op
        WHEN "plus"      
            LET r = a + b
            CALL root_out.setAttribute("r",r)
        WHEN "minus"    
            LET r = a - b
            CALL root_out.setAttribute("r",r)
        WHEN "multiply" 
            LET r = a * b
            CALL root_out.setAttribute("r",r)
        WHEN "divide"  
            IF b != 0 THEN
                LET r = a / b
                LET m = a MOD b
                CALL root_out.setAttribute("r",r)
                CALL root_out.setAttribute("m",m)
            ELSE
                CALL root_out.setAttribute("error","Divide By Zero")
            END IF
    END CASE
END FUNCTION
